"""Generate codes."""
import string

# https://realpython.com/lessons/cryptographically-secure-random-data-python/
import secrets


def get_code(code_length, mode="ascii"):
    """
    Generate a single random code.

    Parameters
    ----------
    code_length : int
        The length of the code

    mode : string
        'ascii' for letters (default), 'digits' for digits and 'combo'
        for combination of letters and digits.

    Return
    ------
    code : string
        The code
    """
    code = ""
    i = 0

    if mode == "ascii":
        characters = string.ascii_uppercase
    elif mode == "digits":
        characters = string.digits
    elif mode == "combo":
        characters = string.ascii_uppercase + string.digits
    else:
        raise ValueError(
            "Invalid value for argument 'mode': " "'{}'".format(mode)
        )

    while i < code_length:
        letter = secrets.choice(characters)
        code += letter
        i += 1
    return code


def get_code_set(count, code_length, mode="ascii"):
    """
    Generate a set of unique, random codes.

    Parameters
    ----------
    count : int
        Number of codes to be returned

    code_length : int
        The length of each code

    mode : string
        'ascii' for letters (default), 'digits' for digits and 'combo'
        for combination of letters and digits.

    Return
    ------
    codes : set
        Set of unique codes
    """
    codes = set()

    while len(codes) < count:
        codes.add(get_code(code_length, mode))

    return codes


def get_code_length_needed(number_of_entries):
    """
    Get code length needed to support one unique code for each entry.

    Parameters
    ----------
    number_of_entries : int
        Number of codewords

    Return
    ------
    code_length: int
        Lenght of code
    """
    # Minimum code length 2
    code_length = 2
    while len(string.ascii_uppercase) ** code_length < number_of_entries:
        code_length = code_length + 1

    return code_length
